@@ -11,11 +11,11 @@ pub mod lang;
 
 use crate::addressing::{Address, Addressable};
 use crate::database::constants::{IS_OF_TYPE_ATTR, TYPE_ADDR, TYPE_HAS_ATTR, TYPE_INVARIANT};
-use crate::database::entry::{Entry, EntryValue};
+use crate::database::entry::{Entry, EntryValue, ImmutableEntry};
 use crate::database::inner::models;
 use crate::database::inner::schema::data;
 use crate::database::lang::Query;
-use crate::util::hash::{Hash, Hashable};
+use crate::util::hash::Hash;
 use crate::util::LoggerSink;
 use anyhow::{anyhow, Result};
 use chrono::NaiveDateTime;
@@ -75,7 +75,7 @@ pub struct OpenResult {
 pub struct UpEndDatabase {
     pool: DbPool,
     pub vault_path: Arc<PathBuf>,
-    pub db_path: Arc<PathBuf>
+    pub db_path: Arc<PathBuf>,
 }
 
 pub const UPEND_SUBDIR: &str = ".upend";
@@ -116,7 +116,7 @@ impl UpEndDatabase {
         let db = UpEndDatabase {
             pool,
             vault_path: Arc::new(PathBuf::from(dirpath.as_ref())),
-            db_path: Arc::new(upend_path)
+            db_path: Arc::new(upend_path),
         };
         let connection = db.connection().unwrap();
 
@@ -140,8 +140,8 @@ impl UpEndDatabase {
 
         trace!("Initializing types...");
         connection.insert_entry(Entry::try_from(&*TYPE_INVARIANT)?)?;
-        upend_insert_addr!(connection, TYPE_ADDR, IS_OF_TYPE_ATTR, TYPE_ADDR);
-        upend_insert_val!(connection, TYPE_ADDR, TYPE_HAS_ATTR, TYPE_HAS_ATTR);
+        upend_insert_addr!(connection, TYPE_ADDR, IS_OF_TYPE_ATTR, TYPE_ADDR)?;
+        upend_insert_val!(connection, TYPE_ADDR, TYPE_HAS_ATTR, TYPE_HAS_ATTR)?;
 
         initialize_hier(&connection)?;
 
@@ -249,6 +249,24 @@ impl UpEndConnection {
             .execute(&self.conn)?)
     }
 
+    pub fn retrieve_entry(&self, hash: Hash) -> Result<Option<Entry>> {
+        use crate::database::inner::schema::data::dsl::*;
+
+        let entry = data
+            .filter(identity.eq(Address::Hash(hash).encode()?))
+            .load::<models::Entry>(&self.conn)?;
+
+        match entry.len() {
+            0 => Ok(None),
+            1 => Ok(Some(Entry::try_from(entry.get(0).unwrap())?)),
+            _ => {
+                unreachable!(
+                    "Multiple entries returned with the same hash - this should be impossible!"
+                )
+            }
+        }
+    }
+
     pub fn retrieve_object(&self, object_address: Address) -> Result<Vec<Entry>> {
         use crate::database::inner::schema::data::dsl::*;
 
@@ -318,23 +336,44 @@ impl UpEndConnection {
 
     pub fn insert_entry(&self, entry: Entry) -> Result<Address> {
         debug!("Inserting: {}", entry);
+        let db_entry = models::Entry::try_from(&entry)?;
+        self._insert_entry(db_entry)?;
+        entry.address()
+    }
 
-        let insert_entry = models::Entry::try_from(&entry)?;
-
-        let entry = Entry::try_from(&insert_entry)?;
+    pub fn insert_entry_immutable(&self, entry: Entry) -> Result<Address> {
+        debug!("Inserting immutably: {}", entry);
+        let address = entry.address()?;
+        let db_entry = models::Entry::try_from(&ImmutableEntry(entry))?;
+        self._insert_entry(db_entry)?;
+        Ok(address)
+    }
 
+    fn _insert_entry(&self, entry: models::Entry) -> Result<usize> {
         let result = diesel::insert_into(data::table)
-            .values(insert_entry)
+            .values(&entry)
             .execute(&self.conn);
 
-        if let Some(error) = result.err() {
-            match error {
-                Error::DatabaseError(DatabaseErrorKind::UniqueViolation, _) => {}
-                _ => return Err(anyhow!(error)),
-            }
+        match result {
+            Ok(num) => Ok(num),
+            Err(error) => match error {
+                Error::DatabaseError(DatabaseErrorKind::UniqueViolation, _) => Ok(0),
+                _ => Err(anyhow!(error)),
+            },
         }
+    }
 
-        Ok(Address::Hash(entry.hash()?))
+    // #[deprecated]
+    pub fn get_all_attributes(&self) -> Result<Vec<String>> {
+        use crate::database::inner::schema::data::dsl::*;
+
+        let result = data
+            .select(attribute)
+            .distinct()
+            .order_by(attribute)
+            .load::<String>(&self.conn)?;
+
+        Ok(result)
     }
 }
 
