@@ -40,15 +40,19 @@ lazy_static! {
 fn initialize_types(connection: &UpEndConnection) -> Result<()> {
     // BLOB_TYPE
     connection.insert_entry(Entry::try_from(&*BLOB_TYPE_INVARIANT)?)?;
-    upend_insert_addr!(connection, BLOB_TYPE_ADDR, IS_OF_TYPE_ATTR, TYPE_ADDR);
-    upend_insert_val!(connection, BLOB_TYPE_ADDR, TYPE_HAS_ATTR, FILE_MTIME_KEY);
-    upend_insert_val!(connection, BLOB_TYPE_ADDR, TYPE_HAS_ATTR, FILE_SIZE_KEY);
-    upend_insert_val!(connection, BLOB_TYPE_ADDR, TYPE_HAS_ATTR, FILE_MIME_KEY);
+    upend_insert_addr!(connection, BLOB_TYPE_ADDR, IS_OF_TYPE_ATTR, TYPE_ADDR)?;
+    upend_insert_val!(connection, BLOB_TYPE_ADDR, TYPE_HAS_ATTR, FILE_MTIME_KEY)?;
+    upend_insert_val!(connection, BLOB_TYPE_ADDR, TYPE_HAS_ATTR, FILE_SIZE_KEY)?;
+    upend_insert_val!(connection, BLOB_TYPE_ADDR, TYPE_HAS_ATTR, FILE_MIME_KEY)?;
 
     Ok(())
 }
 
-pub async fn rescan_vault(db: Arc<UpEndDatabase>, job_container: Arc<RwLock<JobContainer>>) {
+pub async fn rescan_vault(
+    db: Arc<UpEndDatabase>,
+    job_container: Arc<RwLock<JobContainer>>,
+    initial: bool,
+) {
     let job_id = job_container
         .write()
         .unwrap()
@@ -57,7 +61,8 @@ pub async fn rescan_vault(db: Arc<UpEndDatabase>, job_container: Arc<RwLock<JobC
 
     let job_container_rescan = job_container.clone();
     let result =
-        actix_web::web::block(move || _rescan_vault(db, job_container_rescan, job_id)).await;
+        actix_web::web::block(move || _rescan_vault(db, job_container_rescan, job_id, initial))
+            .await;
 
     if result.is_err() {
         let err = result.err().unwrap();
@@ -99,6 +104,7 @@ fn _rescan_vault<D: Borrow<UpEndDatabase>>(
     db: D,
     job_container: Arc<RwLock<JobContainer>>,
     job_id: JobId,
+    initial: bool,
 ) -> Result<Vec<UpdatePathOutcome>> {
     let start = Instant::now();
     info!("Vault rescan started.");
@@ -111,9 +117,12 @@ fn _rescan_vault<D: Borrow<UpEndDatabase>>(
     initialize_types(&connection)?;
 
     // Disable syncing in SQLite for the duration of the import
-    debug!("Disabling SQLite synchronous mode");
-    connection.execute("PRAGMA synchronous = OFF;")?;
-    let _guard = PragmaSynchronousGuard(&connection);
+    let mut _guard: Option<PragmaSynchronousGuard> = None;
+    if initial {
+        debug!("Disabling SQLite synchronous mode");
+        connection.execute("PRAGMA synchronous = OFF;")?;
+        _guard = Some(PragmaSynchronousGuard(&connection));
+    }
 
     // Walk through the vault, find all paths
     debug!("Traversing vault directory");
@@ -181,12 +190,21 @@ fn _rescan_vault<D: Borrow<UpEndDatabase>>(
         }
     });
 
+    // Re-enable SQLite syncing
+    drop(_guard);
+
+    // Reporting
+    let all_outcomes = path_outcomes
+        .into_iter()
+        .chain(cleanup_results)
+        .collect::<Vec<UpdatePathOutcome>>();
+
     let mut failed: Vec<(&PathBuf, &Error)> = vec![];
     let mut created = 0;
     let mut unchanged = 0;
     let mut deleted = 0;
 
-    for outcome in &path_outcomes {
+    for outcome in &all_outcomes {
         match outcome {
             UpdatePathOutcome::Added(_) => created += 1,
             UpdatePathOutcome::Unchanged(_) => unchanged += 1,
@@ -207,9 +225,6 @@ fn _rescan_vault<D: Borrow<UpEndDatabase>>(
         )
     }
 
-    // Re-enable SQLite syncing
-    drop(_guard);
-
     info!(
         "Finished updating {:?} ({} created, {} deleted, {} left unchanged). Took {}s.",
         db.vault_path,
@@ -219,7 +234,7 @@ fn _rescan_vault<D: Borrow<UpEndDatabase>>(
         start.elapsed().as_secs()
     );
 
-    Ok(path_outcomes.into_iter().chain(cleanup_results).collect())
+    Ok(all_outcomes)
 }
 
 fn _process_directory_entry<P: AsRef<Path>>(
@@ -348,8 +363,8 @@ fn _process_directory_entry<P: AsRef<Path>>(
     connection.transaction::<_, Error, _>(|| {
         connection.insert_file(new_file)?;
 
-        connection.insert_entry(type_entry)?;
-        connection.insert_entry(size_entry)?;
+        connection.insert_entry_immutable(type_entry)?;
+        connection.insert_entry_immutable(size_entry)?;
         if let Some(mime_entry) = mime_entry {
             connection.insert_entry(mime_entry)?;
         }
@@ -412,7 +427,7 @@ mod test {
             .unwrap();
 
         // Initial scan
-        let rescan_result = _rescan_vault(&open_result.db, job_container.clone(), job_id);
+        let rescan_result = _rescan_vault(&open_result.db, job_container.clone(), job_id, true);
 
         assert!(rescan_result.is_ok());
         let rescan_result = rescan_result.unwrap();
@@ -423,7 +438,7 @@ mod test {
 
         // Modification-less rescan
 
-        let rescan_result = _rescan_vault(&open_result.db, job_container.clone(), job_id);
+        let rescan_result = _rescan_vault(&open_result.db, job_container.clone(), job_id, false);
 
         assert!(rescan_result.is_ok());
         let rescan_result = rescan_result.unwrap();
@@ -436,7 +451,7 @@ mod test {
 
         std::fs::remove_file(temp_dir.path().join("hello-world.txt")).unwrap();
 
-        let rescan_result = _rescan_vault(&open_result.db, job_container, job_id);
+        let rescan_result = _rescan_vault(&open_result.db, job_container, job_id, false);
 
         assert!(rescan_result.is_ok());
         let rescan_result = rescan_result.unwrap();
