@@ -1,4 +1,4 @@
-import type { IEntry, ListingResult, VALUE_TYPE } from "./types";
+import type { IEntry, IValue, ListingResult, VALUE_TYPE } from "./types";
 
 // export function listingAsOrdered(listing: ListingResult): OrderedListing {
 //   const entries = Object.entries(listing) as [Address, IEntry][];
@@ -103,7 +103,7 @@ export class UpObject {
 export class UpEntry extends UpObject implements IEntry {
   entity: string;
   attribute: string;
-  value: { t: VALUE_TYPE; c: string | number };
+  value: IValue;
 
   constructor(address: string, entry: IEntry, listing: UpListing) {
     super(address, listing);
